package com.robsws.gameoflife;

public class Grid {
	
	int width;
	int height;
	private boolean[][] cells;
	
	/** Constructor that randomises cells to begin with **/
	public Grid(int width, int height) {
		cells = new boolean[width][height];
		this.width = width;
		this.height = height;
		for(int x = 0 ; x < width ; x++) {
			for(int y = 0 ; y < height ; y++) {
				double rnd = Math.random();
				boolean alive = false;
				if(rnd > 0.8) {
					alive = true;
				}
				cells[x][y] = alive;
			}
		}
	}
	
	/** Simulates the next round of the game of life **/
	public void nextRound() {
		boolean[][] newcells = new boolean[width][height];
		
		//Calculate new values
		for(int x = 0 ; x < width ; x++) {
			for(int y = 0 ; y < height ; y++) {
				newcells[x][y] = determineFate(x, y);
			}
		}
		
		//Copy across results
		cells = newcells;
	}
	
	public boolean getCell(int x, int y) {
		return cells[x][y];
	}
	
	/* Apply rules to one cell */
	private boolean determineFate(int x, int y) {
		
		boolean alive = cells[x][y];
		//Count number of living neighbours
		int livingNeighbours = 0;
		for(int i = x - 1 ; i <= x + 1 ; i++) {
			for(int j = y - 1 ; j <= y + 1 ; j++) {
				if(cells[i][j] && !(i == x && j == y)) {
					livingNeighbours++;
				}
			}
		}
		
		//apply rules
		if(alive) {
			if(livingNeighbours < 2 || livingNeighbours > 3) {
				return false;
			} else {
				return true;
			}
		} else {
			if(livingNeighbours == 3) {
				return true;
			} else {
				return false;
			}
		}
	}
}
